import React, { useState, useCallback } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, scaleOrdinal, max, format, extent } from 'd3';
import ReactDropdown from 'react-dropdown';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { useData } from './useData';
import { Marks } from './Marks';
import { Dropdown } from './Dropdown';
import { ColorLegend } from './ColorLegend';
import { SizeLegend } from './SizeLegend';

const width = 960;
const legWidth = 120;
const menuHeight = 80;
const height = 500 - menuHeight;
const margin = { top: 20, right: 30+legWidth, bottom: 65, left: 90 };
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 45;
const fadeOpacity = 0.2;
const normalOpacity = 0.5;
const highlightOpacity = 0.8;

const attributes = [
  { value: 'medianValue', label: 'Median Price' },
  { value: 'crim', label: 'Crime Rate' },
  { value: 'zn', label: 'Land Zoned' },
  { value: 'industry', label: 'Industry' },
  { value: 'charles', label: 'Charles' },
  { value: 'nox', label: 'NO Gas' },
  { value: 'room', label: 'Rooms' },
  { value: 'age', label: 'Old Age' },
  { value: 'distance', label: 'Distance' },
  { value: 'radial', label: 'Radial HWay' },
  { value: 'tax', label: 'Tax Rate' },
  { value: 'ptratio', label: 'P/T Ratio' },
  { value: 'black', label: 'Black' },
  { value: 'lowStat', label: 'Low Status' },
];

const getLabel = value => {
  for (let i = 0; i < attributes.length; i++) {
    if (attributes[i].value === value) {
      return attributes[i].label;
    }
  }
};

const App = () => {
  const data = useData();
  const [hoveredValue, setHoveredValue] = useState(null);
  const [hoveredSizeValue, setHoveredSizeValue] = useState(null);

  const initialXAttribute = 'lowStat';
  const [xAttribute, setXAttribute] = useState(initialXAttribute);
  const xValue = d => d[xAttribute];
  const xAxisLabel = getLabel(xAttribute);

  const initialYAttribute = 'medianValue';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  const yValue = d => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute);

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
  
  const colorValue = d => d.charles;
//  const sizeValue = d => d.radial;
  const sizeValue = d => d.ptratio;
  const tooltipValue = d => d.medianValue;
  const colorLegendLabel = 'Charles';
	const sizeLegendLabel = 'P/T Ratio';

// Update due to hover
	let filteredData = data.filter(d => hoveredValue === colorValue(d));
	const filteredSizeData = data.filter(d => hoveredSizeValue === sizeValue(d));
	const circleRadius = 5;
  if(filteredData.length==0 && filteredSizeData.length!=0)
    filteredData = filteredSizeData;
  
	const hoverOpacity = (hoveredValue !== null | hoveredSizeValue) ? 
        fadeOpacity : (hoveredSizeValue ? highlightOpacity : normalOpacity);

  const siFormat = format('.2s');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0]);

  const sizeScale = scaleLinear()
    .domain(extent(data, sizeValue))
    .range([4, 8]);
  
  const colorScale = scaleOrdinal()
  	.domain(data.map(colorValue))
  	.range(['#E6842A','#137880']); //,'#8E6C8A'
  
  return (
    <>
      <div className="menus-container">
        <span className="dropdown-label">X</span>
        <ReactDropdown
          options={attributes}
          value={xAttribute}
          onChange={({ value }) => setXAttribute(value)}
        />
        <span className="dropdown-label">Y</span>
        <ReactDropdown
          options={attributes}
          value={yAttribute}
          onChange={({ value }) => setYAttribute(value)}
        />
      </div>
      <svg width={width} height={height}>
        <g transform={`translate(${margin.left},${margin.top})`}>
          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            tickFormat={xAxisTickFormat}
            tickOffset={5}
          />
          <text
            className="axis-label"
            textAnchor="middle"
            transform={`translate(${-yAxisLabelOffset},${innerHeight /
              2}) rotate(-90)`}
          >
            {yAxisLabel}
          </text>
          <AxisLeft yScale={yScale} innerWidth={innerWidth} tickOffset={5} />
          <text
            className="axis-label"
            x={innerWidth / 2}
            y={innerHeight + xAxisLabelOffset}
            textAnchor="middle"
          >
            {xAxisLabel}
          </text>
          <g transform={`translate(${innerWidth + 30}, 60)`}>
            <text x={55} y={-25} className="axis-label" textAnchor="middle">
              {colorLegendLabel}
            </text>
            <ColorLegend
              tickSpacing={22}
              tickSize={10}
              tickTextOffset={12}
              tickSize={circleRadius}
              colorScale={colorScale}
              onHover={setHoveredValue}
              hoveredValue={hoveredValue}
              fadeOpacity={fadeOpacity}
            />
          </g>
          <g transform={`translate(${innerWidth + 30}, 160)`}>
            <text x={55} y={-25} className="axis-label" textAnchor="middle">
              {sizeLegendLabel}
            </text>
            <SizeLegend
              tickSpacing={22}
              tickColor={'#E6842A'}
              tickTextOffset={12}
              tickSize={circleRadius}
              sizeScale={sizeScale}
              onHover={setHoveredSizeValue}
              hoveredValue={hoveredSizeValue}
              fadeOpacity={fadeOpacity}
            />
          </g>
          <Marks opacity={hoverOpacity}
            data={data}
            xScale={xScale}
            yScale={yScale}
            xValue={xValue}
            yValue={yValue}
            clrScale={colorScale}
            clrValue={colorValue}
            sizeScale={sizeScale} 
            circleRadius={sizeValue}
            opacity={hoverOpacity}
            tooltipFormat={xAxisTickFormat}
            ttpValue={tooltipValue}
          />
          <Marks
            data={filteredData}
            xScale={xScale}
            yScale={yScale}
            xValue={xValue}
            yValue={yValue}
            clrScale={colorScale}
            clrValue={colorValue}
            sizeScale={sizeScale} 
            circleRadius={sizeValue}
            opacity={hoveredSizeValue ? highlightOpacity : normalOpacity}
            tooltipFormat={xAxisTickFormat}
            ttpValue={tooltipValue}
          />
        </g>
      </svg>
    </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
